{-# LANGUAGE OverloadedStrings #-}

module Oervenster
  ( module OervensterDS
  , Oervenster.nieuw
  , plaatsBoekenoverzicht
  , toonVoorkeuren
  , toonOver
  , meldOnaangekoppeldToestel
  , meldGeslaagdeOverheveling
  ) where


-------------------
--- Eigen steunwaar
-------------------
import           Allerhande
import           AllerhandeGtk
import           Begrippen
import           Boek
import           Boekenoverzicht
import           Hertaling
import           Instellingen
import           Overvenstertje
import           Voorkeurenvenstertje


------------------------
--- Steunwaar van derden
------------------------
import qualified Data.Text    as T
import           GI.Gtk
import           GI.Gio          (VolumeMonitor, volumeMonitorGet, Mount,
                                  FileMonitor)
import           System.FilePath (takeDirectory, takeBaseName)


------------------------
--- Eigen datastructuren
------------------------
import OervensterDS


-------
-- Code
-------
nieuw :: IO Oervenster
nieuw = do
  bwr <- bouwerNieuw glade
  v   <- venster bwr
  bon <- toestellenstapel bwr
  tn  <- toestellen bwr
  zi  <- zeefinvoer bwr
  zb  <- zeefbalk bwr
  zk  <- zeefknop bwr
  mk  <- meldingenknop bwr
  mp  <- meldingenpaneel bwr
  mh  <- meldingenhouder bwr

  schwr <- volumeMonitorGet -- schouwen naar toestellen
  instn <- raadpleegInstellingen

  let toestand = Oervenster bwr v bon tn zi zb zk mk mp mh schwr instn
  verbindZoekbalk toestand
  verbindMeldingenpaneel toestand

  geef toestand


toonOver :: Oervenster -> IO ()
toonOver = Overvenstertje.trachtTeTonen

toonVoorkeuren :: Oervenster -> IO ()
toonVoorkeuren = Voorkeurenvenstertje.trachtTeTonen


plaatsBoekenoverzicht toestand overzicht naam = do
  let stapel = _toestellenstapel toestand
  let toner  = _schuiver overzicht
  stackAddTitled stapel toner naam naam


verbindMeldingenpaneel :: Oervenster -> IO ()
verbindMeldingenpaneel toestand = do
  let paneel = (_meldingenpaneel toestand)
  let knop   = (_meldingenknop toestand)
    
  onPopoverClosed paneel (toggleButtonSetActive knop False >> leegte)
  onToggleButtonToggled knop wisselZichtbaarheid

  widgetShowAll paneel
  leegte
 where wisselZichtbaarheid = do
         ingedrukt <- getToggleButtonActive (_meldingenknop toestand)
         if ingedrukt
         then toonMeldingenpaneel (_meldingenknop toestand)
                                  (_meldingenpaneel toestand)
         else popoverPopdown $ _meldingenpaneel toestand
         leegte


verbindZoekbalk :: Oervenster -> IO ()
verbindZoekbalk toestand = do
  searchBarConnectEntry (_zeefbalk toestand) (_zeefinvoer toestand)
  onToggleButtonToggled (_zeefknop toestand) wisselZichtbaarheid

  sneltoetsen  <- accelGroupNew
  (toets, mod) <- acceleratorParse "<Control>f"
  windowAddAccelGroup (_oervenster toestand) sneltoetsen
  widgetAddAccelerator (_zeefknop toestand) "activate" sneltoetsen toets mod []

  leegte
 where wisselZichtbaarheid = do
         ingedrukt <- getToggleButtonActive (_zeefknop toestand)
         searchBarSetSearchMode (_zeefbalk toestand) ingedrukt
--  onWidgetKeyPressEvent (_oervenster toestand) (afhandelaar balk)
--  leegte
-- where afhandelaar :: SearchBar -> Gdk.EventKey -> IO Bool
--       afhandelaar balk eventKey = do
--         event <- Gdk.unsafeCastTo Gdk.Event eventKey
--         searchBarHandleEvent balk eventKey


-- Meldingenpaneel -------------------------------------------------------------

toonMeldingenpaneel knop paneel = do
  stĳl <- widgetGetStyleContext knop
  styleContextRemoveClass stĳl "suggested-action"
  popoverPopup paneel


meldOnaangekoppeldToestel :: Oervenster -> Tekst -> IO ()
meldOnaangekoppeldToestel toestand naam = do
  bouwer   <- bouwerNieuw gladeMeldingen
  melding  <- meldingOnaankoppelbaar bouwer
  toestel  <- meldingOnaankoppelbaarToestel bouwer
  
  labelSetText toestel naam
  
  let houder = _meldingenhouder toestand
  boxPackEnd houder melding True False 0
  verbergNietsTeMelden toestand
  widgetShowAll melding
  meldMelding toestand


meldGeslaagdeOverheveling :: Oervenster -> Boek -> BestandspadT -> IO ()
meldGeslaagdeOverheveling toestand boek doel = do
  bouwer   <- bouwerNieuw gladeMeldingen
  melding  <- meldingOvergeheveld bouwer
  schrĳver <- meldingOvergeheveldSchrĳver bouwer
  titel    <- meldingOvergeheveldTitel    bouwer
  toestel  <- meldingOvergeheveldToestel  bouwer

  labelSetText schrĳver $ boekAuteur' boek
  labelSetText titel    $ boekTitel'  boek
  labelSetText toestel  $ T.pack $ takeBaseName $ takeDirectory $ T.unpack doel

  let houder = _meldingenhouder toestand
  boxPackEnd houder melding True False 0
  verbergNietsTeMelden toestand
  widgetShowAll melding
  meldMelding toestand


meldMislukteOverheveling :: IO ()
meldMislukteOverheveling = undefined


verbergNietsTeMelden :: Oervenster -> IO ()
verbergNietsTeMelden toestand = do
  nietstemelden' <- nietstemelden $ _bouwer toestand
  widgetHide nietstemelden'


meldMelding :: Oervenster -> IO ()
meldMelding toestand = do 
  let knop = _meldingenknop toestand
  stĳl <- widgetGetStyleContext knop
  styleContextAddClass stĳl "suggested-action"
  leegte


-- Glade -----------------------------------------------------------------------

glade = begrMiddelen <> "gtk/oervenster.ui"
gladeMeldingen = begrMiddelen <> "gtk/meldingen.ui"

-- Wanneer een dezer onderdelen niet geladen kan worden is de fundamentele
-- werking van het programma ondergraven. Boekerĳ mag dan crashen en er is
-- daarom geen reden dit veilig te doen
venster          = laadGtk "oervenster"          ApplicationWindow
toestellenstapel = laadGtk "verzamelingenstapel" Stack
toestellen       = laadGtk "leest_toestellen"    ListStore
zeefinvoer       = laadGtk "zeefinvoer"          SearchEntry
zeefbalk         = laadGtk "zeefbalk"            SearchBar
zeefknop         = laadGtk "zeefknop"            ToggleButton
meldingenknop    = laadGtk "meldingenknop"       ToggleButton
meldingenpaneel  = laadGtk "meldingenpaneel"     Popover
meldingenhouder  = laadGtk "meldingenhouder"     Box
nietstemelden    = laadGtk "nietstemelden"       Label

meldingOvergeheveld         = laadGtk "overgeheveld"          Box
meldingOvergeheveldSchrĳver = laadGtk "overgeheveld_schrĳver" Label
meldingOvergeheveldTitel    = laadGtk "overgeheveld_titel"    Label
meldingOvergeheveldToestel  = laadGtk "overgeheveld_toestel"  Label

meldingOnaankoppelbaar        = laadGtk "onaankoppelbaar"         Box
meldingOnaankoppelbaarToestel = laadGtk "onaankoppelbaar_toestel" Label
